// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KQUICKIMAGEEDITOR_VERSION_H
#define KQUICKIMAGEEDITOR_VERSION_H

#define KQUICKIMAGEEDITOR_VERSION_STRING "0.6.0"
#define KQUICKIMAGEEDITOR_VERSION_MAJOR 0
#define KQUICKIMAGEEDITOR_VERSION_MINOR 6
#define KQUICKIMAGEEDITOR_VERSION_PATCH 0
#define KQUICKIMAGEEDITOR_VERSION ((0<<16)|(6<<8)|(0))

#endif
