
#ifndef KQUICKIMAGEEDITOR_EXPORT_H
#define KQUICKIMAGEEDITOR_EXPORT_H

#ifdef KQUICKIMAGEEDITOR_STATIC_DEFINE
#  define KQUICKIMAGEEDITOR_EXPORT
#  define KQUICKIMAGEEDITOR_NO_EXPORT
#else
#  ifndef KQUICKIMAGEEDITOR_EXPORT
#    ifdef KQuickImageEditor_EXPORTS
        /* We are building this library */
#      define KQUICKIMAGEEDITOR_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KQUICKIMAGEEDITOR_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KQUICKIMAGEEDITOR_NO_EXPORT
#    define KQUICKIMAGEEDITOR_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KQUICKIMAGEEDITOR_DECL_DEPRECATED
#  define KQUICKIMAGEEDITOR_DECL_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KQUICKIMAGEEDITOR_DECL_DEPRECATED_EXPORT
#  define KQUICKIMAGEEDITOR_DECL_DEPRECATED_EXPORT KQUICKIMAGEEDITOR_EXPORT KQUICKIMAGEEDITOR_DECL_DEPRECATED
#endif

#ifndef KQUICKIMAGEEDITOR_DECL_DEPRECATED_NO_EXPORT
#  define KQUICKIMAGEEDITOR_DECL_DEPRECATED_NO_EXPORT KQUICKIMAGEEDITOR_NO_EXPORT KQUICKIMAGEEDITOR_DECL_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KQUICKIMAGEEDITOR_NO_DEPRECATED
#    define KQUICKIMAGEEDITOR_NO_DEPRECATED
#  endif
#endif

#define KQUICKIMAGEEDITOR_DECL_DEPRECATED_TEXT(text) KQUICKIMAGEEDITOR_DECL_DEPRECATED

#if defined(KQUICKIMAGEEDITOR_NO_DEPRECATED)
#  undef KQUICKIMAGEEDITOR_DEPRECATED
#  define KQUICKIMAGEEDITOR_DEPRECATED_EXPORT KQUICKIMAGEEDITOR_EXPORT
#  define KQUICKIMAGEEDITOR_DEPRECATED_NO_EXPORT KQUICKIMAGEEDITOR_NO_EXPORT
#elif defined(KQUICKIMAGEEDITOR_NO_DEPRECATED_WARNINGS)
#  define KQUICKIMAGEEDITOR_DEPRECATED
#  define KQUICKIMAGEEDITOR_DEPRECATED_EXPORT KQUICKIMAGEEDITOR_EXPORT
#  define KQUICKIMAGEEDITOR_DEPRECATED_NO_EXPORT KQUICKIMAGEEDITOR_NO_EXPORT
#else
#  define KQUICKIMAGEEDITOR_DEPRECATED KQUICKIMAGEEDITOR_DECL_DEPRECATED
#  define KQUICKIMAGEEDITOR_DEPRECATED_EXPORT KQUICKIMAGEEDITOR_DECL_DEPRECATED_EXPORT
#  define KQUICKIMAGEEDITOR_DEPRECATED_NO_EXPORT KQUICKIMAGEEDITOR_DECL_DEPRECATED_NO_EXPORT
#endif

/* No deprecated API had been removed from build */
#define KQUICKIMAGEEDITOR_EXCLUDE_DEPRECATED_BEFORE_AND_AT 0

#define KQUICKIMAGEEDITOR_BUILD_DEPRECATED_SINCE(major, minor) 1

#ifdef KQUICKIMAGEEDITOR_NO_DEPRECATED
#  define KQUICKIMAGEEDITOR_DISABLE_DEPRECATED_BEFORE_AND_AT 0x600
#endif
#ifdef KQUICKIMAGEEDITOR_NO_DEPRECATED_WARNINGS
#  define KQUICKIMAGEEDITOR_DEPRECATED_WARNINGS_SINCE 0
#endif

#ifndef KQUICKIMAGEEDITOR_DEPRECATED_WARNINGS_SINCE
#  ifdef KQUICKIMAGEEDITOR_DISABLE_DEPRECATED_BEFORE_AND_AT
#    define KQUICKIMAGEEDITOR_DEPRECATED_WARNINGS_SINCE KQUICKIMAGEEDITOR_DISABLE_DEPRECATED_BEFORE_AND_AT
#  else
#    define KQUICKIMAGEEDITOR_DEPRECATED_WARNINGS_SINCE 0x600
#  endif
#endif

#ifndef KQUICKIMAGEEDITOR_DISABLE_DEPRECATED_BEFORE_AND_AT
#  define KQUICKIMAGEEDITOR_DISABLE_DEPRECATED_BEFORE_AND_AT 0x0
#endif

#ifdef KQUICKIMAGEEDITOR_DEPRECATED
#  define KQUICKIMAGEEDITOR_ENABLE_DEPRECATED_SINCE(major, minor) (((major<<16)|(minor<<8)) > KQUICKIMAGEEDITOR_DISABLE_DEPRECATED_BEFORE_AND_AT)
#else
#  define KQUICKIMAGEEDITOR_ENABLE_DEPRECATED_SINCE(major, minor) 0
#endif

#endif /* KQUICKIMAGEEDITOR_EXPORT_H */
